@@ -30,7 +30,7 @@
 #include "orbit/orbit.h"
 
 #define NUM_RUNS 1
-#define NUM_THREADS 0
+#define NUM_THREADS 8
 
 #undef TIMING_RUN
 
@@ -1388,16 +1388,13 @@ testMisc (test_TestFactory   factory,
 	} else { /* Lets do some things on a de-activated ref */
 		test_BasicServer         obj;
 		PortableServer_ObjectId *oid;
-		POA_test_BasicServer     servant = {
-			NULL, &BasicServer_vepv
-		};
+		POA_test_BasicServer    *servant;
 
-		obj = create_object (
-			global_poa, POA_test_BasicServer__init,
-			&servant, ev);
+		servant = SIMPLE_SERVANT_NEW (BasicServer);
+		obj = create_object (global_poa, servant, ev);
 
 		oid = PortableServer_POA_servant_to_id (
-			global_poa, &servant, ev);
+			global_poa, servant, ev);
 		g_assert (ev->_major == CORBA_NO_EXCEPTION);
 		PortableServer_POA_deactivate_object (
 			global_poa, oid, ev);
@@ -1584,19 +1581,27 @@ testAsync (test_TestFactory   factory,
 }
 
 static void
-broken_cb (LINCConnection *connection, gboolean *broken)
+broken_cb (LinkConnection *connection, gboolean *broken)
 {
 	*broken = TRUE;
 }
 
 static void
-testPingPong (test_TestFactory   factory, 
+testPingPong (test_TestFactory   factory,
+	      gboolean           thread_tests,
 	      CORBA_Environment *ev)
 {
 	test_PingPongServer r_objref, l_objref, objref;
 	CORBA_unsigned_long before_remote_hash;
 	CORBA_unsigned_long after_remote_hash;
 
+#if NUM_THREADS > 0
+	if (thread_tests) {
+		g_warning ("No thread available to handle incoming requests");
+		return;
+	}
+#endif
+
 	d_print ("Testing ping pong invocations ...\n");
 	r_objref = test_TestFactory_createPingPongServer (factory, ev);
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
@@ -1717,7 +1722,7 @@ testPingPong (test_TestFactory   factory,
 }
 
 static void
-dummy_cb (LINCConnection *connection, gboolean *invoked)
+dummy_cb (LinkConnection *connection, gboolean *invoked)
 {
 	*invoked = TRUE;
 }
@@ -1819,7 +1824,7 @@ test_time_noop (test_TestFactory   factory,
 	g_timer_elapsed (timer, NULL) * 1000);
 
 	old_flags = ORBit_small_flags;
-	ORBit_small_flags &= ~ORBIT_SMALL_FAST_LOCALS;
+	ORBit_small_flags != ORBIT_SMALL_FORCE_GENERIC_MARSHAL;
 	g_timer_reset (timer);
 	g_timer_start (timer);
 	for (i = 0; i < TIME_TEST_RUNS; i++)
@@ -1856,9 +1861,48 @@ test_basic_server (test_TestFactory   factory,
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 }
 
+static void
+testDerivedServer (test_TestFactory   factory, 
+		   CORBA_Environment *ev)
+{
+	CORBA_Object            obj;
+	PortableServer_ServantBase *servant;
+	PortableServer_ObjectId *oid;
+	ORBit_POAObject             pobj;
+	PortableServer_ServantBase__epv DerivedServer_base_epv = {NULL, simple_finalize, NULL};
+	POA_test_BasicServer__vepv      DerivedServer_vepv = { &DerivedServer_base_epv, NULL };
+
+	d_print ("Testing DerivedServer ...\n");
+
+	servant = SIMPLE_SERVANT_NEW (DerivedServer);
+	obj = create_object (global_poa, servant, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	g_assert (test_C1__classid != 0);
+	g_assert (test_B1__classid != 0);
+	g_assert (test_B2__classid != 0);
+	g_assert (test_DerivedServer__classid != 0);
+
+	pobj = (ORBit_POAObject) obj->adaptor_obj;
+	g_assert (pobj->vepvmap_cache [test_DerivedServer__classid] != 0);
+	g_assert (pobj->vepvmap_cache [test_C1__classid] != 0);
+	g_assert (pobj->vepvmap_cache [test_B1__classid] != 0);
+	g_assert (pobj->vepvmap_cache [test_B2__classid] != 0);
+
+	CORBA_Object_release (obj, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+
+	oid = PortableServer_POA_servant_to_id (global_poa, servant, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	PortableServer_POA_deactivate_object (global_poa, oid, ev);
+	g_assert (ev->_major == CORBA_NO_EXCEPTION);
+	
+	CORBA_free (oid);	
+}
+
 static void
 run_tests (test_TestFactory   factory, 
-	   gboolean           reenterant_only,
+	   gboolean           thread_tests,
 	   CORBA_Environment *ev)
 {
 	int i;
@@ -1890,12 +1934,13 @@ run_tests (test_TestFactory   factory,
 		testTypeCode (factory, ev);
 		testContext (factory, ev);
 		testIInterface (factory, ev);
+		testDerivedServer (factory, ev);
 #ifndef TIMING_RUN
-		if (!reenterant_only)
+		if (!thread_tests)
 			testAsync (factory, ev);
 #endif
 		if (!in_proc)
-			testPingPong (factory, ev);
+			testPingPong (factory, thread_tests, ev);
 		testMisc (factory, ev);
 	}
 	
@@ -1948,9 +1993,9 @@ static void
 dump_protos (void)
 {
 	int enabled_count = 0;
-	LINCProtocolInfo *info;
+	LinkProtocolInfo *info;
 
-	for (info = linc_protocol_all (); info->name; info++) {
+	for (info = link_protocol_all (); info->name; info++) {
 		gboolean enabled;
 
 		if ((enabled = ORBit_proto_use (info->name)))
@@ -1982,7 +2027,8 @@ main (int argc, char *argv [])
 	test_TestFactory   factory;
 	ORBit_IInterfaces *interfaces = NULL;
 	gboolean           gen_imodule = FALSE;
-	gboolean           threaded = FALSE;
+	gboolean           thread_safe = FALSE;
+	gboolean           thread_tests = FALSE;
 	const char        *orb_name;
 	int                i;
 
@@ -1997,11 +2043,15 @@ main (int argc, char *argv [])
 	for (i = 0; i < argc; i++) {
 		if (!strcmp (argv [i], "--gen-imodule"))
 			gen_imodule = TRUE;
-		if (!strcmp (argv [i], "--threaded"))
-			threaded = TRUE;
+		if (!strcmp (argv [i], "--thread-safe"))
+			thread_safe = TRUE;
+		if (!strcmp (argv [i], "--thread-tests")) {
+			thread_safe = TRUE;
+			thread_tests = TRUE;
+		}
 	}
 
-	if (threaded)
+	if (thread_safe)
 		orb_name = "orbit-local-mt-orb";
 	else
 		orb_name = "orbit-local-orb";
@@ -2009,6 +2059,11 @@ main (int argc, char *argv [])
 	global_orb = CORBA_ORB_init (&argc, argv, orb_name, ev);
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
 
+	/*	if (thread_tests) {
+		g_warning ("FIXME: testing only");
+		link_set_io_thread (TRUE);
+		} */
+
 	if (gen_imodule) {
 		CORBA_sequence_CORBA_TypeCode *typecodes = NULL;
 
@@ -2039,8 +2094,9 @@ main (int argc, char *argv [])
 
 	test_time_noop (factory, ev);
 	run_tests (factory, FALSE, ev);
-	if (threaded)
-		run_threaded_tests (factory, ev);
+	if (thread_tests)
+		g_warning ("FIXME: disabled in-proc threaded tests for now");
+/*		run_threaded_tests (factory, ev); */
 
 	CORBA_Object_release (factory, ev);
 	g_assert (ev->_major == CORBA_NO_EXCEPTION);
@@ -2069,7 +2125,7 @@ main (int argc, char *argv [])
 		g_assert (ev->_major == CORBA_NO_EXCEPTION);
 	}
 	run_tests (factory, FALSE, ev);
-	if (threaded)
+	if (thread_tests)
 		run_threaded_tests (factory, ev);
 	testSegv (factory, ev);
 
